@@ -31,6 +31,7 @@
 # include <string.h>
 # include <ctype.h>
 #endif
+#include <getopt.h>
 #ifdef HAVE_PCRE2_H
 //use both, 8 and 16 (r2007+)
 # define PCRE2_CODE_UNIT_WIDTH 0
@@ -38,13 +39,9 @@
 #endif
 
 #include "dwg.h"
-#include "../src/logging.h"
-#include "../src/common.h"
-#include "../src/bits.h"
-#include "suffix.inc"
-static int help(void);
-int verbosity(int argc, char **argv, int i, unsigned int *opts);
-#include "common.inc"
+#include "logging.h"
+#include "common.h"
+#include "bits.h"
 #include "dwg_api.h"
 
 #ifndef HAVE_PCRE2_H
@@ -123,15 +120,21 @@ static int help(void) {
 #if 0
   printf("  -R, -r, --recursive       Recursively search subdirectories listed.\n");
 #endif
-  printf("  --type NAME               Search only NAME entities or objects.\n");
-  printf("  --dxf NUM                 Search only DXF group NUM fields.\n");
-  printf("  --text                    Search only in TEXT-like entities.\n");
+  printf("  -y, --type NAME           Search only NAME entities or objects.\n");
+  printf("  -d, --dxf NUM             Search only DXF group NUM fields.\n");
+  printf("  -t, --text                Search only in TEXT-like entities.\n");
 #if 0
-  printf("  --tables                  Search only in table names.\n");
+  printf("  -b, --tables              Search only in table names.\n");
 #endif
+#ifdef HAVE_GETOPT_LONG
   printf("      --help                Display this help and exit\n");
   printf("      --version             Output version information and exit\n"
          "\n");
+#else
+  printf("  -u                        Display this help and exit\n");
+  printf("  -v                        Output version information and exit\n"
+         "\n");
+#endif
   printf("GNU LibreDWG online manual: <https://www.gnu.org/software/libredwg/>\n");
   return 0;
 }
@@ -172,7 +175,7 @@ do_match (const int is16, const char *restrict filename,
       print_match(is16, filename, entity, dxfgroup, text);
     return 1;
   } else if (rc < -2) { //not PCRE2_ERROR_NOMATCH nor PCRE2_ERROR_PARTIAL
-    pcre2_get_error_message_8(rc, buf, 4096);
+    pcre2_get_error_message_8(rc, (PCRE2_UCHAR8 *)buf, 4096);
     LOG_WARN("pcre2 match error %s with %s", buf, pattern);
   }
   return 0;
@@ -237,8 +240,8 @@ do_match (const int is16, const char *restrict filename,
 #define MATCH_DXF(type,ENTITY,text_field,dxfgroup)  \
   if (numdxf) { \
     int dxfok = 0; \
-    for (int i=0; i<numdxf; i++) { \
-      if (dxf[i] == dxfgroup) { dxfok = 1; break; } \
+    for (int _i=0; _i<numdxf; _i++) { \
+      if (dxf[_i] == dxfgroup) { dxfok = 1; break; } \
     } \
     if (dxfok) { MATCH_TYPE(type,ENTITY,text_field,dxfgroup); } \
   } \
@@ -248,7 +251,7 @@ do_match (const int is16, const char *restrict filename,
 
 //8bit only
 #define MATCH_NO16(type,ENTITY,text_field,dxfgroup) \
-  text = obj->tio.type->tio.ENTITY->text_field; \
+  text = (char*)obj->tio.type->tio.ENTITY->text_field; \
   if (text && numdxf) { \
     int dxfok = 0; \
     for (int i=0; i<numdxf; i++) { \
@@ -264,12 +267,12 @@ do_match (const int is16, const char *restrict filename,
 
 #ifdef HAVE_PCRE2_16
 #define MATCH_TYPE(type,ENTITY,text_field,dxfgroup)  \
-  text = obj->tio.type->tio.ENTITY->text_field; \
+  text = (char*)obj->tio.type->tio.ENTITY->text_field; \
   if (text) \
     found += do_match(obj->parent->header.version >= R_2007, filename, #ENTITY, dxfgroup, text)
 #else
 #define MATCH_TYPE(type,ENTITY,text_field,dxfgroup)  \
-  text = obj->tio.type->tio.ENTITY->text_field; \
+  text = (char*)obj->tio.type->tio.ENTITY->text_field; \
   if (text) { \
     if (obj->parent->header.version >= R_2007) \
       text = bit_convert_TU((BITCODE_TU)text); \
@@ -362,17 +365,27 @@ int match_VIEWPORT(const char *restrict filename, const Dwg_Object *restrict obj
 static
 int match_3DSOLID(const char *restrict filename, const Dwg_Object *restrict obj)
 {
-  char *text;
-  int found = 0, i;
+  char *text = NULL;
+  int found = 0;
+  BITCODE_BL j;
   Dwg_Entity_3DSOLID *_obj = obj->tio.entity->tio._3DSOLID;
 
-  for (i=0; i<_obj->num_blocks; i++)
+  if (!_obj || !obj->tio.entity) return 0;
+  if (_obj->acis_data) {
+    MATCH_NO16 (entity, _3DSOLID, acis_data, 1);
+    //MATCH_ENTITY (_3DSOLID, acis_data, 1);
+    //found += do_match(0, filename, "3DSOLID", 1, (char*)_obj->acis_data);
+  }
+  /*
+  if (!_obj->encr_sat_data) return 0;
+  for (j=0; j<_obj->num_blocks; j++)
     {
-      text = obj->tio.entity->tio._3DSOLID->encr_sat_data[i];
-      if (text)
-        found += do_match(0, filename, "3DSOLID", 301, text);
-      //MATCH_NO16 (entity, _3DSOLID, encr_sat_data[i], 301);
+      //text = _obj->encr_sat_data[j];
+      //if (text)
+      //  found += do_match(0, filename, "3DSOLID", 301, text);
+      MATCH_NO16 (entity, _3DSOLID, encr_sat_data[j], 301);
     }
+  */
   return found;
 }
 
@@ -380,7 +393,8 @@ static
 int match_DICTIONARY(const char *restrict filename, const Dwg_Object *restrict obj)
 {
   char *text;
-  int found = 0, i;
+  int found = 0;
+  BITCODE_BL i;
   Dwg_Object_DICTIONARY *_obj = obj->tio.object->tio.DICTIONARY;
 
   for (i=0; i<_obj->numitems; i++)
@@ -473,10 +487,22 @@ int match_IMAGEDEF(const char *restrict filename, const Dwg_Object *restrict obj
 }
 
 static
-int match_LAYER_INDEX(const char *restrict filename, const Dwg_Object *restrict obj)
+int match_SCALE(const char *restrict filename, const Dwg_Object *restrict obj)
 {
   char *text;
   int found = 0, i;
+  const Dwg_Object_SCALE *_obj = obj->tio.object->tio.SCALE;
+
+  MATCH_OBJECT (SCALE, name, 1);
+  return found;
+}
+
+static
+int match_LAYER_INDEX(const char *restrict filename, const Dwg_Object *restrict obj)
+{
+  char *text;
+  int found = 0;
+  BITCODE_BL i;
   const Dwg_Object_LAYER_INDEX *_obj = obj->tio.object->tio.LAYER_INDEX;
 
   for (i=0; i<_obj->num_entries; i++)
@@ -490,7 +516,8 @@ static
 int match_LAYOUT(const char *restrict filename, const Dwg_Object *restrict obj)
 {
   char *text;
-  int found = 0, i;
+  int found = 0;
+  BITCODE_BL i;
   const Dwg_Object_LAYOUT *_obj = obj->tio.object->tio.LAYOUT;
 
   MATCH_OBJECT (LAYOUT, page_setup_name, 1);
@@ -514,7 +541,8 @@ static
 int match_FIELD(const char *restrict filename, const Dwg_Object *restrict obj)
 {
   char *text;
-  int found = 0, i;
+  int found = 0;
+  BITCODE_BL i;
   const Dwg_Object_FIELD *_obj = obj->tio.object->tio.FIELD;
 
   MATCH_OBJECT (FIELD, format, 4);
@@ -531,11 +559,31 @@ int match_FIELD(const char *restrict filename, const Dwg_Object *restrict obj)
   return found;
 }
 
+static
+int match_TABLE(const char *restrict filename, const Dwg_Object *restrict obj)
+{
+  char *text;
+  int found = 0;
+  BITCODE_BL i;
+  const Dwg_Entity_TABLE *_obj = obj->tio.entity->tio.TABLE;
+
+  for (i=0; i<_obj->num_cells; i++)
+    {
+      if (_obj->cells[i].type == 1) {
+        MATCH_ENTITY (TABLE, cells[i].text_string, 1);
+      } else if (_obj->cells[i].type == 2 && _obj->cells[i].additional_data_flag == 1) {
+        MATCH_ENTITY (TABLE, cells[i].attr_def_text, 300);
+      }
+    }
+  return found;
+}
+
 static
 int match_TABLECONTENT(const char *restrict filename, const Dwg_Object *restrict obj)
 {
   char *text;
-  int found = 0, i, j, k;
+  int found = 0;
+  BITCODE_BL i, j, k;
   const Dwg_Object_TABLECONTENT *_obj = obj->tio.object->tio.TABLECONTENT;
 
   MATCH_OBJECT (TABLECONTENT, ldata.name, 1);
@@ -577,7 +625,7 @@ static
 int match_GEODATA(const char *restrict filename, const Dwg_Object *restrict obj)
 {
   char *text;
-  int found = 0, i;
+  int found = 0;
   const Dwg_Object_GEODATA *_obj = obj->tio.object->tio.GEODATA;
 
   MATCH_OBJECT (GEODATA, coord_system_def, 0);
@@ -591,6 +639,156 @@ int match_GEODATA(const char *restrict filename, const Dwg_Object *restrict obj)
   return found;
 }
 
+static
+int match_GEOPOSITIONMARKER(const char *restrict filename, const Dwg_Object *restrict obj)
+{
+  char *text;
+  int found = 0;
+  const Dwg_Entity_GEOPOSITIONMARKER *_obj = obj->tio.entity->tio.GEOPOSITIONMARKER;
+
+  MATCH_ENTITY (GEOPOSITIONMARKER, text, 1);
+  MATCH_ENTITY (GEOPOSITIONMARKER, notes, 3);
+  return found;
+}
+
+static
+int match_UNDERLAYDEFINITION(const char *restrict filename, const Dwg_Object *restrict obj)
+{
+  char *text;
+  int found = 0;
+  const Dwg_Object_UNDERLAYDEFINITION *_obj = obj->tio.object->tio.UNDERLAYDEFINITION;
+
+  MATCH_OBJECT (UNDERLAYDEFINITION, filename, 1);
+  MATCH_OBJECT (UNDERLAYDEFINITION, name, 2);
+  return found;
+}
+
+static
+int match_VISUALSTYLE(const char *restrict filename, const Dwg_Object *restrict obj)
+{
+  char *text;
+  int found = 0, i;
+  const Dwg_Object_VISUALSTYLE *_obj = obj->tio.object->tio.VISUALSTYLE;
+
+  MATCH_OBJECT (VISUALSTYLE, desc, 1);
+  return found;
+}
+
+static
+int match_LIGHT(const char *restrict filename, const Dwg_Object *restrict obj)
+{
+  char *text;
+  int found = 0;
+  const Dwg_Entity_LIGHT *_obj = obj->tio.entity->tio.LIGHT;
+
+  MATCH_ENTITY (LIGHT, name, 1);
+  //MATCH_ENTITY (LIGHT, web_file, 1);
+  return found;
+}
+
+static
+int match_SUNSTUDY(const char *restrict filename, const Dwg_Object *restrict obj)
+{
+  char *text;
+  int found = 0;
+  const Dwg_Object_SUNSTUDY *_obj = obj->tio.object->tio.SUNSTUDY;
+
+  MATCH_OBJECT (SUNSTUDY, setup_name, 1);
+  MATCH_OBJECT (SUNSTUDY, desc, 2);
+  MATCH_OBJECT (SUNSTUDY, sheet_set_name, 3);
+  return found;
+}
+
+static
+int match_LIGHTLIST(const char *restrict filename, const Dwg_Object *restrict obj)
+{
+  char *text;
+  int found = 0, i;
+  const Dwg_Object_LIGHTLIST *_obj = obj->tio.object->tio.LIGHTLIST;
+
+  for (i=0; i<_obj->num_lights; i++)
+    {
+      //MATCH_OBJECT (LIGHTLIST, lights[i].name, 1);
+    }
+  return found;
+}
+
+static
+int match_DBCOLOR(const char *restrict filename, const Dwg_Object *restrict obj)
+{
+  char *text;
+  int found = 0;
+  const Dwg_Object_DBCOLOR *_obj = obj->tio.object->tio.DBCOLOR;
+
+  MATCH_OBJECT (DBCOLOR, name, 430);
+  MATCH_OBJECT (DBCOLOR, catalog, 430);
+  return found;
+}
+
+static
+int match_MATERIAL(const char *restrict filename, const Dwg_Object *restrict obj)
+{
+  char *text;
+  int found = 0;
+  const Dwg_Object_MATERIAL *_obj = obj->tio.object->tio.MATERIAL;
+
+  MATCH_OBJECT (MATERIAL, name, 1);
+  MATCH_OBJECT (MATERIAL, desc, 2);
+  MATCH_OBJECT (MATERIAL, diffusemap_filename, 3);
+  MATCH_OBJECT (MATERIAL, specularmap_filename, 4);
+  MATCH_OBJECT (MATERIAL, reflectionmap_filename, 6);
+  MATCH_OBJECT (MATERIAL, opacitymap_filename, 7);
+  MATCH_OBJECT (MATERIAL, bumpmap_filename, 8);
+  MATCH_OBJECT (MATERIAL, refractionmap_filename, 9);
+  MATCH_OBJECT (MATERIAL, normalmap_filename, 3);
+  MATCH_OBJECT (MATERIAL, genprocname, 300);
+  MATCH_OBJECT (MATERIAL, genprocvaltext, 301);
+  MATCH_OBJECT (MATERIAL, genprocvalcolorname, 430);
+  return found;
+}
+
+static
+int match_PLOTSETTINGS(const char *restrict filename, const Dwg_Object *restrict obj)
+{
+  char *text;
+  int found = 0;
+  const Dwg_Object_PLOTSETTINGS *_obj = obj->tio.object->tio.PLOTSETTINGS;
+
+  MATCH_OBJECT (PLOTSETTINGS, page_setup_name, 1);
+  MATCH_OBJECT (PLOTSETTINGS, printer_cfg_file, 2);
+  MATCH_OBJECT (PLOTSETTINGS, paper_size, 4);
+  return found;
+}
+
+static
+int match_ASSOCACTION(const char *restrict filename, const Dwg_Object *restrict obj)
+{
+  char *text;
+  int found = 0;
+  const Dwg_Object_ASSOCACTION *_obj = obj->tio.object->tio.ASSOCACTION;
+  MATCH_OBJECT (ASSOCACTION, body.evaluatorid, 0);
+  MATCH_OBJECT (ASSOCACTION, body.expresssion, 0);
+  return found;
+}
+static
+int match_ASSOCOSNAPPOINTREFACTIONPARAM(const char *restrict filename, const Dwg_Object *restrict obj)
+{
+  char *text;
+  int found = 0;
+  const Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM *_obj = obj->tio.object->tio.ASSOCOSNAPPOINTREFACTIONPARAM;
+  MATCH_OBJECT (ASSOCOSNAPPOINTREFACTIONPARAM, name, 1);
+  return found;
+}
+static
+int match_ACDBNAVISWORKSMODELDEF(const char *restrict filename, const Dwg_Object *restrict obj)
+{
+  char *text;
+  int found = 0;
+  const Dwg_Object_ACDBNAVISWORKSMODELDEF *_obj = obj->tio.object->tio.ACDBNAVISWORKSMODELDEF;
+  MATCH_OBJECT (ACDBNAVISWORKSMODELDEF, path, 1);
+  return found;
+}
+
 static
 int match_BLOCK_HEADER(const char *restrict filename, Dwg_Object_Ref *restrict ref)
 {
@@ -679,12 +877,40 @@ int match_BLOCK_HEADER(const char *restrict filename, Dwg_Object_Ref *restrict r
                 found += match_LAYER_INDEX(filename, obj);
               else if (obj->type == DWG_TYPE_LAYOUT)
                 found += match_LAYOUT(filename, obj);
+              else if (obj->fixedtype == DWG_TYPE_SCALE)
+                found += match_SCALE(filename, obj);
               else if (obj->fixedtype == DWG_TYPE_FIELD)
                 found += match_FIELD(filename, obj);
+              else if (obj->fixedtype == DWG_TYPE_TABLE)
+                found += match_TABLE(filename, obj);
               else if (obj->fixedtype == DWG_TYPE_TABLECONTENT)
                 found += match_TABLECONTENT(filename, obj);
               else if (obj->fixedtype == DWG_TYPE_GEODATA)
                 found += match_GEODATA(filename, obj);
+              else if (obj->fixedtype == DWG_TYPE_GEOPOSITIONMARKER)
+                found += match_GEOPOSITIONMARKER(filename, obj);
+              else if (obj->fixedtype == DWG_TYPE_UNDERLAYDEFINITION)
+                found += match_UNDERLAYDEFINITION(filename, obj);
+              else if (obj->fixedtype == DWG_TYPE_VISUALSTYLE)
+                found += match_VISUALSTYLE(filename, obj);
+              else if (obj->fixedtype == DWG_TYPE_LIGHT)
+                found += match_LIGHT(filename, obj);
+              else if (obj->fixedtype == DWG_TYPE_SUNSTUDY)
+                found += match_SUNSTUDY(filename, obj);
+              else if (obj->fixedtype == DWG_TYPE_LIGHTLIST)
+                found += match_LIGHTLIST(filename, obj);
+              else if (obj->fixedtype == DWG_TYPE_DBCOLOR)
+                found += match_DBCOLOR(filename, obj);
+              else if (obj->fixedtype == DWG_TYPE_MATERIAL)
+                found += match_MATERIAL(filename, obj);
+              else if (obj->fixedtype == DWG_TYPE_PLOTSETTINGS)
+                found += match_PLOTSETTINGS(filename, obj);
+              else if (obj->fixedtype == DWG_TYPE_ASSOCACTION)
+                found += match_ASSOCACTION(filename, obj);
+              else if (obj->fixedtype == DWG_TYPE_ASSOCOSNAPPOINTREFACTIONPARAM)
+                found += match_ASSOCOSNAPPOINTREFACTIONPARAM(filename, obj);
+              else if (obj->fixedtype == DWG_TYPE_ACDBNAVISWORKSMODELDEF)
+                found += match_ACDBNAVISWORKSMODELDEF(filename, obj);
             }
         }
       if (!opt_text)
@@ -734,69 +960,97 @@ main (int argc, char *argv[])
 #endif
   int opt_recurse = 0;
   int count = 0;
+  int c;
+#ifdef HAVE_GETOPT_LONG
+  int option_index = 0;
+  static struct option long_options[] = {
+        {"case",      0, 0, 'i'},
+        {"extended",  0, 0, 'x'},
+        {"count",     0, 0, 'c'},
+        {"no-filename",0, 0, 'h'},
+        {"recursive", 0, 0, 'r'},
+        {"recursive", 0, 0, 'R'},
+        {"type",      1, 0, 'y'},
+        {"dxf",       1, 0, 'd'},
+        {"text",      0, 0, 't'},
+        {"tables",    0, 0, 'b'},
+        {"help",      0, 0, 0},
+        {"version",   0, 0, 0},
+        {NULL,        0, NULL, 0}
+  };
+#endif
 
   // check args
   if (argc < 2)
     return usage();
-
   memset(dxf, 0, 10*sizeof(short));
-  if (i < argc && !strcmp(argv[i], "-i"))
-    {
-      options |= PCRE2_CASELESS;
-      i++;
-    }
-#ifdef HAVE_PCRE2_H
-  if (i < argc && !strcmp(argv[i], "-x"))
-    {
-      options |= PCRE2_EXTENDED;
-      i++;
-    }
-#endif
-  if (i < argc && (!strcmp(argv[i], "--count") || !strcmp(argv[i], "-c")))
-    {
-      opt_count = 1;
-      i++;
-    }
-  if (i < argc && (!strcmp(argv[i], "--no-filename") || !strcmp(argv[i], "-h")))
-    {
-      opt_filename = 0;
-      i++;
-    }
-  if (i < argc && (!strcmp(argv[i], "--recursive") ||
-                   !strcmp(argv[i], "-R") ||
-                   !strcmp(argv[i], "-r")))
-    {
-      opt_recurse = 1;
-      i++;
-    }
-  if (i < argc-1 && !strcmp(argv[i], "--type"))
-    {
-      if (numtype >= 10) return usage(); //too many
-      type[numtype++] = argv[i+1]; // a string
-      i += 2;
-    }
-  if (i < argc-1 && !strcmp(argv[i], "--dxf"))
-    {
-      if (numdxf >= 10) return usage(); // too many
-      // a integer group
-      dxf[numdxf++] = (short)strtol(argv[i+1], NULL, 10);
-      i += 2;
-    }
-  if (i < argc && !strcmp(argv[i], "--text"))
-    {
-      opt_text = 1;
-      i++;
-    }
-  if (i < argc && !strcmp(argv[i], "--tables"))
-    {
-      opt_tables = 1;
-      i++;
-    }
-  if (i < argc && !strcmp(argv[i], "--help"))
-    return help();
-  if (i < argc && !strcmp(argv[i], "--version"))
-    return opt_version();
 
+  while
+#ifdef HAVE_GETOPT_LONG
+    ((c = getopt_long(argc, argv, "ixchrRy:d:tb",
+                      long_options, &option_index)) != -1)
+#else
+    ((c = getopt(argc, argv, "ixchrRy:d:tbvu")) != -1)
+#endif
+    {
+      if (c == -1) break;
+      switch (c) {
+#ifdef HAVE_GETOPT_LONG
+      case 0:
+        if (!strcmp(long_options[option_index].name, "help"))
+          return help();
+        if (!strcmp(long_options[option_index].name, "version"))
+          return opt_version();
+        break;
+#else
+      case 'v':
+        return opt_version();
+#endif
+#ifdef HAVE_PCRE2_H
+      case 'x':
+        options |= PCRE2_EXTENDED;
+        break;
+#endif
+      case 'i':
+        options |= PCRE2_CASELESS;
+        break;
+      case 'c':
+        opt_count = 1;
+        break;
+      case 'h':
+        opt_filename = 0;
+        break;
+      case 'r':
+      case 'R':
+        opt_recurse = 1;
+        break;
+      case 't':
+        opt_text = 1;
+        break;
+      case 'b':
+        opt_tables = 1;
+        break;
+      case 'y':
+        if (numtype >= 10) return usage(); //too many
+        type[numtype++] = optarg; // a string
+        break;
+      case 'd':
+        if (numdxf >= 10) return usage(); // too many
+        // a integer group
+        dxf[numdxf++] = (short)strtol(optarg, NULL, 10);
+        break;
+
+      case 'u':
+        return help();
+      case '?':
+        fprintf(stderr, "%s: invalid option '-%c' ignored\n",
+                argv[0], optopt);
+        break;
+      default:
+        return usage();
+      }
+    }
+  i = optind;
   if (i > argc-2) // need 2 more args. TODO: unless -R given
     return usage();
 
@@ -815,7 +1069,7 @@ main (int argc, char *argv[])
 # endif
     );
   if (errcode != 0 && errcode != 100) {
-    pcre2_get_error_message_8(errcode, buf, 4096);
+    pcre2_get_error_message_8(errcode, (PCRE2_UCHAR8 *)buf, 4096);
     LOG_ERROR("pcre2_compile_8 error %d: %s with %s", errcode, buf, pattern);
     return 1;
   }
@@ -838,7 +1092,7 @@ main (int argc, char *argv[])
 #  endif
     );
   if (errcode != 0 && errcode != 100) {
-    pcre2_get_error_message_8(errcode, buf, 4096);
+    pcre2_get_error_message_8(errcode, (PCRE2_UCHAR8 *)buf, 4096);
     LOG_ERROR("pcre2_compile_16 error %d: %s with %s", errcode, buf, pattern);
     return 1;
   }
