@@ -28,7 +28,7 @@ Add CLASSES for those
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
-#include <math.h>
+//#include <math.h>
 
 #include "common.h"
 #include "bits.h"
@@ -84,6 +84,7 @@ static int dxf_3dsolid(Bit_Chain *restrict dat,
     fprintf(dat->fh, "\r\n"); \
   }
 #endif
+#define VALUE_TFF(str,dxf)    VALUE_TV(str, dxf)
 #define VALUE_BINARY(value,size,dxf) \
 { \
   long len = size; \
@@ -162,32 +163,33 @@ static int dxf_3dsolid(Bit_Chain *restrict dat,
 /* avoid empty numbers, and fixup some bad %f libc formatting */
 #define VALUE(value, type, dxf) \
   if (dxf) { \
-    char *s; \
-    const char *fmt = dxf_format (dxf); \
+    char *_s; \
+    const char *_fmt = dxf_format (dxf); \
     GROUP(dxf); \
     GCC_DIAG_IGNORE(-Wformat-nonliteral) \
-    snprintf(buf, 255, fmt, value); \
+    snprintf(buf, 255, _fmt, value); \
     GCC_DIAG_RESTORE \
     /* not a string, emtpy num. must be zero */ \
-    if (strcmp(fmt, "%s") && !*buf) \
+    if (strcmp(_fmt, "%s") && !*buf) \
       strcpy(buf, "0"); \
-    else if (90 <= dxf && dxf < 100) \
-      /* ignore -Wpointer-to-int-cast, cannot happen with 90 */ \
-      snprintf(buf, 255, "%6i", (int32_t)(value)); \
-    else if (!strcmp(fmt, "%-16.14f")) { \
+    else if (90 <= dxf && dxf < 100) { \
+      /* -Wpointer-to-int-cast */ \
+      const int32_t _si = (int32_t)(intptr_t)(value); \
+      snprintf(buf, 255, "%6i", _si); \
+    } else if (!strcmp(_fmt, "%-16.14f")) {      \
       if (!strcmp(buf, "0.00000000000000")) \
         strcpy(buf, "0.0"); \
-      else if ((s = strstr(buf, ".00000000000000"))) \
-        strcpy(s, ".0"); \
-      else if ((s = strstr(buf, ".50000000000000"))) \
-        strcpy(s, ".5"); \
-      else if ((s = strstr(buf, ".12500000000000"))) \
-        strcpy(s, ".125"); \
+      else if ((_s = strstr(buf, ".00000000000000"))) \
+        strcpy(_s, ".0"); \
+      else if ((_s = strstr(buf, ".50000000000000"))) \
+        strcpy(_s, ".5"); \
+      else if ((_s = strstr(buf, ".12500000000000"))) \
+        strcpy(_s, ".125"); \
     } \
     fprintf(dat->fh, "%s\r\n", buf); \
   }
 #define VALUE_RD(value, dxf) \
-  if (dxf && !isnan(value)) { \
+  if (dxf && !bit_isnan(value)) { \
     GROUP(dxf); \
     if (value == 0.0 || value == 0) \
       fprintf(dat->fh, "0.0\r\n"); \
@@ -224,7 +226,8 @@ static int dxf_3dsolid(Bit_Chain *restrict dat,
 #define HEADER_RL(name,dxf)  HEADER_9(name); FIELD(name, RL, dxf)
 #define HEADER_RLL(name,dxf) HEADER_9(name); FIELD(name, RLL, dxf)
 #define HEADER_TV(name,dxf)  HEADER_9(name); VALUE_TV(_obj->name, dxf)
-#define HEADER_T(name,dxf)   HEADER_9(name); VALUE_T(_obj->name, dxf)
+#define HEADER_TU(name,dxf)  HEADER_9(name); VALUE_TU(_obj->name, dxf)
+#define HEADER_T(name,dxf)   HEADER_9(name); VALUE_T((char*)_obj->name, dxf)
 #define HEADER_B(name,dxf)   HEADER_9(name); FIELD_B(name, dxf)
 #define HEADER_BS(name,dxf)  HEADER_9(name); FIELD(name, BS, dxf)
 #define HEADER_BD(name,dxf)  HEADER_9(name); FIELD_BD(name, dxf)
@@ -291,8 +294,12 @@ static int dxf_3dsolid(Bit_Chain *restrict dat,
 #define FIELD_3BD(name,dxf) {FIELD_BD(name.x, dxf); FIELD_BD(name.y, dxf+10); FIELD_BD(name.z, dxf+20);}
 #define FIELD_3BD_1(name,dxf) {FIELD_BD(name.x, dxf); FIELD_BD(name.y, dxf+1); FIELD_BD(name.z, dxf+2);}
 #define FIELD_3DPOINT(name,dxf) FIELD_3BD(name,dxf)
-#define FIELD_CMC(name,dxf)\
-  VALUE_RS(_obj->name.index, dxf)
+#define FIELD_CMC(color,dxf1,dxf2) { \
+  VALUE_RS(_obj->color.index, dxf1); \
+  if (dat->version >= R_2004 && dxf2 && _obj->color.rgb) { \
+    VALUE_RL(_obj->color.rgb, dxf2); \
+  } \
+}
 #define HEADER_TIMEBLL(name, dxf) \
   HEADER_9(name); FIELD_TIMEBLL(name, dxf)
 #define FIELD_TIMEBLL(name,dxf) \
@@ -318,20 +325,20 @@ static int dxf_3dsolid(Bit_Chain *restrict dat,
 // reads data of the type indicated by 'type' 'size' times and stores
 // it all in the vector called 'name'.
 #define FIELD_VECTOR_N(name, type, size, dxf)\
-  if (dxf)\
+  if (dxf && _obj->name)\
     {\
-      for (vcount=0; vcount < (int)size; vcount++)\
+      for (vcount=0; vcount < (BITCODE_BL)size; vcount++)\
         {\
           VALUE_##type(_obj->name[vcount], dxf); \
         }\
     }
 #define FIELD_VECTOR_T(name, size, dxf)\
-  if (dxf) {\
+  if (dxf && _obj->name) {\
     PRE (R_2007) {                                                   \
-      for (vcount=0; vcount < (int)_obj->size; vcount++)             \
+      for (vcount=0; vcount < (BITCODE_BL)_obj->size; vcount++)             \
         VALUE_TV(_obj->name[vcount], dxf);                           \
     } else {                                                         \
-      for (vcount=0; vcount < (int)_obj->size; vcount++)             \
+      for (vcount=0; vcount < (BITCODE_BL)_obj->size; vcount++)             \
         VALUE_TU(_obj->name[vcount], dxf);                           \
     }                                                                \
   }
@@ -339,8 +346,8 @@ static int dxf_3dsolid(Bit_Chain *restrict dat,
 #define FIELD_VECTOR(name, type, size, dxf) FIELD_VECTOR_N(name, type, _obj->size, dxf)
 
 #define FIELD_2RD_VECTOR(name, size, dxf)\
-  if (dxf) {\
-    for (vcount=0; vcount < (int)_obj->size; vcount++)    \
+  if (dxf && _obj->name) {\
+    for (vcount=0; vcount < (BITCODE_BL)_obj->size; vcount++)    \
       {\
         FIELD_2RD(name[vcount], dxf);\
       }\
@@ -348,22 +355,24 @@ static int dxf_3dsolid(Bit_Chain *restrict dat,
 
 #define FIELD_2DD_VECTOR(name, size, dxf)\
   FIELD_2RD(name[0], dxf);\
-  for (vcount = 1; vcount < (int)_obj->size; vcount++)\
-    {\
-      FIELD_2DD(name[vcount], FIELD_VALUE(name[vcount - 1].x), FIELD_VALUE(name[vcount - 1].y), dxf);\
-    }\
+  if (dxf && _obj->name) {\
+    for (vcount = 1; vcount < (BITCODE_BL)_obj->size; vcount++)\
+      {\
+        FIELD_2DD(name[vcount], FIELD_VALUE(name[vcount - 1].x), FIELD_VALUE(name[vcount - 1].y), dxf);\
+      }\
+  }
 
 #define FIELD_3DPOINT_VECTOR(name, size, dxf)\
   if (dxf) {\
-    for (vcount=0; vcount < (int)_obj->size; vcount++)\
+    for (vcount=0; vcount < (BITCODE_BL)_obj->size; vcount++)\
       {\
         FIELD_3DPOINT(name[vcount], dxf);\
       }\
     }
 
 #define VALUE_HANDLE_N(hdlptr, name, vcount, handle_code, dxf) \
-  if (dxf) {\
-    for (vcount=0; vcount < (int)size; vcount++)\
+  if (dxf && hdlptr && size) {\
+    for (vcount=0; vcount < (BITCODE_BL)size; vcount++)\
       {\
         VALUE_HANDLE(hdlptr[vcount], handle_code, dxf);\
       }\
@@ -371,8 +380,8 @@ static int dxf_3dsolid(Bit_Chain *restrict dat,
 #define FIELD_HANDLE_N(name, size, handle_code, dxf) \
   VALUE_HANDLE(_obj->name, handle_code, dxf)
 #define HANDLE_VECTOR_N(name, size, code, dxf) \
-  if (dxf) {\
-    for (vcount=0; vcount < (int)size; vcount++)\
+  if (dxf && _obj->name && size) {\
+    for (vcount=0; vcount < (BITCODE_BL)size; vcount++)\
       {\
         FIELD_HANDLE(name[vcount], code, dxf);\
       }\
@@ -402,7 +411,7 @@ static int dxf_3dsolid(Bit_Chain *restrict dat,
       obj->tio.object->reactors) \
   { \
     fprintf(dat->fh, "102\r\n{ACAD_REACTORS\r\n");\
-    for (vcount=0; vcount < (int)obj->tio.object->num_reactors; vcount++)\
+    for (vcount=0; vcount < obj->tio.object->num_reactors; vcount++)\
       { /* soft ptr */ \
         VALUE_HANDLE(obj->tio.object->reactors[vcount], code, 330); \
       }\
@@ -435,16 +444,25 @@ static int dxf_3dsolid(Bit_Chain *restrict dat,
 #define END_STRING_STREAM
 #define START_HANDLE_STREAM
 
+#ifndef DEBUG_CLASSES
+static int
+dwg_dxf_TABLECONTENT (Bit_Chain *restrict dat, const Dwg_Object *restrict obj)
+{
+  (void)dat; (void)obj;
+  return 0;
+}
+#else
 static int
 dwg_dxf_TABLECONTENT (Bit_Chain *restrict dat, const Dwg_Object *restrict obj);
+#endif
 
 //The strcmp is being optimized away at compile-time! https://godbolt.org/g/AqkhwL
 #define DWG_ENTITY(token) \
 static int \
 dwg_dxf_##token (Bit_Chain *restrict dat, const Dwg_Object *restrict obj) \
 {\
+  BITCODE_BL vcount, rcount1, rcount2, rcount3, rcount4; \
   int error = 0; \
-  long vcount, rcount1, rcount2, rcount3, rcount4; \
   Dwg_Data* dwg = obj->parent; \
   Dwg_Entity_##token *ent, *_obj;\
   Dwg_Object_Entity *_ent;\
@@ -488,8 +506,8 @@ dwg_dxf_##token (Bit_Chain *restrict dat, const Dwg_Object *restrict obj) \
 static int \
 dwg_dxf_ ##token (Bit_Chain *restrict dat, const Dwg_Object *restrict obj) \
 { \
+  BITCODE_BL vcount, rcount1, rcount2, rcount3, rcount4;\
   int error = 0; \
-  long vcount, rcount1, rcount2, rcount3, rcount4;\
   Bit_Chain *hdl_dat = dat;\
   Dwg_Data* dwg = obj->parent; \
   Dwg_Object_##token *_obj;\
@@ -538,7 +556,7 @@ dxf_write_xdata(Bit_Chain *restrict dat, Dwg_Resbuf *restrict rbuf, BITCODE_BL s
           if (type == VT_INVALID) {
             LOG_WARN("Invalid xdata code %d", dxftype);
           } else {
-            LOG_WARN("Unknown xdata code %d => %d", dxftype, (int)type);
+            LOG_WARN("Unknown xdata code %d => %d", dxftype, (BITCODE_BL)type);
           }
         }
 
@@ -590,7 +608,7 @@ dxf_write_xdata(Bit_Chain *restrict dat, Dwg_Resbuf *restrict rbuf, BITCODE_BL s
 }
 
 #undef DXF_3DSOLID
-#define DXF_3DSOLID dxf_3dsolid(dat, obj, _obj);
+#define DXF_3DSOLID dxf_3dsolid(dat, obj, (Dwg_Entity_3DSOLID*)_obj);
 
 // r13+ converts STANDARD to Standard, BYLAYER to ByLayer, BYBLOCK to ByBlock
 static void
@@ -678,9 +696,9 @@ static int dxf_3dsolid(Bit_Chain *restrict dat,
 {
   Dwg_Data* dwg = obj->parent;
   unsigned long j;
-  int vcount, rcount1, rcount2;
+  BITCODE_BL vcount, rcount1, rcount2;
+  BITCODE_BL i;
   int error = 0;
-  int i = 0;
   int index;
   int total_size = 0;
   int num_blocks = 0;
@@ -816,7 +834,10 @@ static int
 dwg_dxf_object(Bit_Chain *restrict dat, const Dwg_Object *restrict obj)
 {
   int error = 0;
-  const int minimal = obj->parent->opts & 0x10;
+  int minimal;
+  if (!obj || !obj->parent)
+    return DWG_ERR_INTERNALERROR;
+  minimal = obj->parent->opts & 0x10;
 
   if (obj->supertype == DWG_SUPERTYPE_UNKNOWN)
     return 0;
@@ -917,6 +938,7 @@ dwg_dxf_object(Bit_Chain *restrict dat, const Dwg_Object *restrict obj)
       return dwg_dxf_MLINE(dat, obj);
 #else
       LOG_WARN("Unhandled Entity MLINE in out_dxf %u/%lX", obj->index, obj->handle.value)
+      if (0) dwg_dxf_MLINE(dat, obj);
       return DWG_ERR_UNHANDLEDCLASS;
 #endif
     case DWG_TYPE_BLOCK_CONTROL:
@@ -1008,13 +1030,58 @@ dxf_common_entity_handle_data(Bit_Chain *restrict dat, const Dwg_Object *restric
   Dwg_Object_Entity *ent;
   //Dwg_Data *dwg = obj->parent;
   Dwg_Object_Entity *_obj;
-  int i, error = 0;
-  long unsigned int vcount = 0;
+  int error = 0;
+  BITCODE_BL vcount = 0;
   ent = obj->tio.entity;
   _obj = ent;
 
   #include "common_entity_handle_data.spec"
 
+#if 1
+  #include "common_entity_data.spec"
+#else
+  if (ent->picture_exists && ent->picture_size >= 0 && ent->picture_size < 210210) {
+    FIELD_RL(picture_size, 160);
+    VALUE_BINARY(ent->picture, ent->picture_size, 310); //chunked hex encoding
+  }
+  SINCE(R_2004) {
+    if (ent->color.index > 0)
+      FIELD_BL(color.index, 62);
+    if (ent->color.flag & 0x80)
+      FIELD_BL(color.rgb & 0x00ffffff, 420);
+    if (ent->color.flag & 0x20) {
+      if (ent->color.transparency_type == 0)
+        VALUE_TV("ByLayer", 440)
+      else if (ent->color.transparency_type == 1)
+        VALUE_TV("ByBlock", 440)
+      else if (ent->color.transparency_type == 3)
+        FIELD_BL(color.alpha, 440)
+    }
+    if ((ent->color.flag & 0x41) == 0x41)
+      FIELD_TV(color.name, 430)
+    if ((ent->color.flag  & 0x42) == 0x42)
+      FIELD_TV(color.book_name, 430)
+  }
+  if (ent->linetype_scale > 0.0)
+    FIELD_BD (linetype_scale, 48)
+
+  SINCE(R_2000)
+    {
+      switch (ent->linetype_flags) {
+      case 1: VALUE_TV("ByBlock", 7); break;
+      case 2: VALUE_TV("CONTINUOUS", 7); break;
+      //case 3: HANDLE_NAME(LTYPE, 7); break;
+      }
+      // 00 BYLAYER, 01 BYBLOCK, 10 CONTINUOUS, 11 plotstyle handle
+      FIELD_BB (plotstyle_flags, 0);
+    }
+  SINCE(R_2007)
+    {
+      FIELD_BB (material_flags, 0); //if not BYLAYER 00: 347 material handle
+      FIELD_RC (shadow_flags, 284);
+    }
+#endif
+
   return error;
 }
 
@@ -1327,8 +1394,6 @@ dxf_tables_write (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
         }
       // avoid unused warnings
       dwg_dxf_PROXY_ENTITY(dat, &dwg->object[0]);
-      dwg_dxf_UNKNOWN_ENT(dat, &dwg->object[0]);
-      dwg_dxf_UNKNOWN_OBJ(dat, &dwg->object[0]);
       ENDTAB();
     }
   SINCE (R_13)
@@ -1356,7 +1421,7 @@ dxf_tables_write (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
     }
     for (i=0; i<dwg->block_control.num_entries; i++)
       {
-        Dwg_Object *obj = dwg_ref_object(dwg, dwg->block_control.block_headers[i]);
+        obj = dwg_ref_object(dwg, dwg->block_control.block_headers[i]);
         if (obj && obj->type == DWG_TYPE_BLOCK_HEADER &&
             obj != mspace && obj != pspace)
           {
@@ -1374,7 +1439,7 @@ static int
 dxf_blocks_write (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
 {
   int error = 0;
-  unsigned int i;
+  //unsigned int i;
   Dwg_Object_BLOCK_CONTROL *_ctrl = &dwg->block_control;
   Dwg_Object *ctrl = &dwg->object[_ctrl->objid];
   /* let's see if this control block is correct... */
@@ -1417,7 +1482,7 @@ dxf_blocks_write (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
     hdr = _ctrl->paper_space->obj;
 
   if (hdr) {
-      Dwg_Object *obj = get_first_owned_block(hdr);
+      obj = get_first_owned_block(hdr);
       while (obj)
         {
           error |= dwg_dxf_object(dat, obj);
@@ -1437,7 +1502,7 @@ static int
 dxf_entities_write (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
 {
   int error = 0;
-  long unsigned int i;
+  BITCODE_BL i;
 
   SECTION(ENTITIES);
   for (i=0; i<dwg->num_objects; i++)
@@ -1455,7 +1520,7 @@ static int
 dxf_objects_write (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
 {
   int error = 0;
-  long unsigned int i;
+  BITCODE_BL i;
 
   SECTION(OBJECTS);
   for (i=0; i<dwg->num_objects; i++)
