@@ -255,8 +255,8 @@ add_lwpolyline (xmlNodePtr rootnode, const Dwg_Object* obj)
   buf = doubletochar (lwpline->elevation);
   newXMLProp ("Elevation", buf);
 
-  buf = spointprepare (lwpline->normal.x, lwpline->normal.y, lwpline->normal.z);
-  newXMLProp ("Normal", buf);
+  buf = spointprepare (lwpline->extrusion.x, lwpline->extrusion.y, lwpline->extrusion.z);
+  newXMLProp ("Extrusion", buf);
 
   newXMLcProp ("ObjectName", "AcDbPolyline");
 
@@ -675,7 +675,7 @@ add_table (xmlNodePtr rootnode, const Dwg_Object *obj)
 int
 load_dwg (char *dwgfilename, xmlNodePtr rootnode)
 {
-  unsigned int i;
+  BITCODE_BL i;
   int error;
   Dwg_Data dwg;
 
